:root {
    --primary: #FDF0D5;
    --white-man: #780000;
    --black-man: #336683;
    --light: #FDF0D5;
    --dark: #669BBC;
    --dark: #003049;
    --secondary: #333;
    --red: #3bce4575;
    --tetriary: #669BBC;
}

/* 
--hunyadi-yellow: #edae49ff;
--amaranth: #d1495bff;
--caribbean-current: #00798cff;
--lapis-lazuli: #30638eff;
--indigo-dye: #003d5bff;

*/

main {
    /* display: flex;
    align-items: center;
    justify-content: center; */
    height: 100vh;
    font-family: sans-serif;
    font-size: 1.5rem;
    background-color: var(--secondary);
    color: var(--light);
    overflow: hidden;
}

.board {
    display: grid;
    grid-template-columns: repeat(10, 1fr);
    grid-template-rows: repeat(10, 1fr);
    grid-gap: 1px;
    border: 2px;
    height: 95%;

    margin: 20px auto;
    background-color: var(--secondary);
    padding: 10px;
    border-radius: 10px;
    color: var(--light);

}

.tile {
    position: relative;
    /* display */
    display: flex;
    /* justify content */
    justify-content: center;
    /* align items */
    align-items: center;
    /* font size */
    font-size: 30px;
    /* font weight */
    color: var(--light);
    /* border */
    border: 2px solid var(--primary);
    /* border radius */
    border-radius: 10px;
    /* background */
    background: var(--primary);
    /* cursor */
    cursor: pointer;
    /* hover-time */
    transition: 0.2s ease-in-out;
    /* not user select */
    user-select: none;
}

.higlight {
    background-color: var(--red) !important;
    border-radius: 70px;
    transition: 0.1s ease-in-out;
}

.dark-tile {
    background-color: var(--dark);
}

.light-tile {
    background-color: var(--light);
    color: var(--dark);
}

.piece {
    position: absolute;
    width: 80%;
    height: 80%;
    border-radius: 50%;
    opacity: .84;
    background: var(--primary);
    box-shadow: 0.3rem 0.9rem 0.7rem rgba(0, 0, 0, 0.747);
    pointer-events: auto;
    z-index: 1;
}



.crown {
    width: 60%;
    height: 60%;
    opacity: 0.4;
    margin: auto;
    position: absolute;
    pointer-events: none;
    align-self: center;
    justify-self: center;

    z-index: 2;
}

.tile:hover {
    /* background */
    border-radius: 6px;
}



.tile:active .piece {
    /* background */
    width: 70%;
    height: 70%;
}

.box {
    background: #fff;
    border-radius: 4px;
    padding-bottom: 100%;
}

.tile-text {
    position: absolute;
    text-align: center;
    font-weight: bolder;

}

.btn-custom {
    background-color: var(--primary);
    color: var(--dark);
    border: 2px solid var(--dark);
    border-radius: 10px;
    font-weight: bold;
    cursor: pointer;
    transition: 0.01s ease-in-out;
    user-select: none;
}

.btn-custom:hover {
    background-color: var(--dark);
    color: var(--primary);
    border: 2px solid var(--primary);
}

.history-container {
    position: relative;
    overflow-y: scroll;
    max-height: 500px;
    background-color: var(--secondary) !important;
    border: 0 2px solid var(--primary);
    border-radius: 10px;
    font-weight: bold;
    cursor: pointer;
}

table {
    color: var(--light) !important;
    border: var(--light) !important;
    font-size: medium;
}

tr {
    border: 1px solid var(--light) !important;
}

/* width */
::-webkit-scrollbar {
    width: 10px;
    /* background: var(--primary); */
    border-radius: 0px;
}

/* Track */
::-webkit-scrollbar-track {
    border-radius: 0px;
    background: var(--secondary);
}

/* Handle */
::-webkit-scrollbar-thumb {
    background: var(--primary);
    /* border-radius: inherit; */
}

/* auto scroll to bottom */
.history-container {
    scroll-behavior: smooth;
}